#-----------------------------------------------------------------------
# R para Data Science
#
#                Prof. Dr. Walmes M. Zeviani & Prof. Dr. Wagner H. Bonat
#                Department of Statistics · Federal University of Paraná
#                                       2021-mai-11 · Curitiba/PR/Brazil
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Download do arquivo ZIP.

# https://www.datacamp.com/community/tutorials/r-data-import-tutorial

url <- "http://leg.ufpr.br/~walmes/data/google-books-R-Python.zip"
dest <- sprintf("~/Downloads/%s", basename(url))
download.file(url = url,
              destfile = dest)

# Lista o conteúdo do arquivo ZIP.
unzip(dest, list = TRUE)

# Decompacta arquivo ZIP.
outdir <- tools::file_path_sans_ext(dest)
unzip(dest, exdir = outdir)

# Confere.
dir.exists(outdir)
list.files(path = outdir)

#-----------------------------------------------------------------------
# Análise dos resultados.

library(tidyverse)
# ls("package:purrr")

fls <- list.files(path = outdir,
                  pattern = "*.json")
head(fls)

gsub("-\\d+\\.json", "", fls) %>%
    table() %>%
    enframe() %>%
    mutate(language = ifelse(str_detect(name, "Python"), "Python", "R")) %>%
    count(language, wt = value)

#-----------------------------------------------------------------------
# Importar os JSON.

library(jsonlite)

# Nome dos arquivos.
fls <- list.files(path = outdir,
                  pattern = "*.json",
                  full.names = TRUE)
head(fls)

# Lista com cada arquivo importado.
# Entra e cada e retém o 3 elemento da lista.
jsons <- map(fls, read_json) %>%
    map(~.$items)

# Confere o tamanho.
length(jsons) == length(fls)

# Remove o primeiro nível de separação da lista.
# jsons <- reduce(jsons, append)
jsons <- unlist(jsons, recursive = FALSE)
length(jsons)

jsons[[1]]

# Cria tabela organizando os livros.
tb_books <- jsons %>%
    # head(3) %>%
    map_dfr(function(el) {
        list(id = el$id,
             title = el$volumeInfo$title,
             date = el$volumeInfo$publishedDate,
             publisher = el$volumeInfo$publisher,
             authors = el$volumeInfo$authors) %>%
            discard(is.null) %>%
            as_tibble()
    })
tb_books

#-----------------------------------------------------------------------
# Confere alguns títulos.

# grepl() = str_detect()
tb_books %>% filter(grepl("r cookbook", title, ignore.case = TRUE))
tb_books %>% filter(grepl("advanced r", title, ignore.case = TRUE))
tb_books %>% filter(grepl("r book", title, ignore.case = TRUE))
tb_books %>% filter(grepl("book of r", title, ignore.case = TRUE))

browseURL("https://www.amazon.com/Primer-Chapman-Hall-CRC/dp/1138631973")
tb_books %>% filter(grepl("r primer", title, ignore.case = TRUE))

browseURL("https://www.amazon.com/Machine-Learning-Factor-Investing-Mathematics/dp/0367545861")
tb_books %>% filter(grepl("r version", title, ignore.case = TRUE))

#-----------------------------------------------------------------------
# Remove duplicações.

# Remove duplicações.
tb_books <- tb_books %>%
    distinct(id, .keep_all = TRUE) %>%
    distinct(str_trim(str_to_upper(title)), .keep_all = TRUE)

with(tb_books,
     str_subset(str_to_upper(title), pattern = "\\b(R\\.|R-|R -)"))

# Mostra títulos que tem R mas são abreviações de nomes, etc.
tb_books %>%
    filter(str_detect(str_to_upper(title), pattern = "\\b(R\\.|R-|R -)")) %>%
    select(title) %>%
    print(n = 30)

#-----------------------------------------------------------------------
# Classifica os livros quanto ao uso das linguagens.

# Seleciona títulos e classifica se contiver R, Python ou ambos.
tb_books <- tb_books %>%
    filter(str_detect(str_to_upper(title),
                      pattern = "\\b(R\\.|R-|R -)",
                      negate = TRUE)) %>%
    mutate(isR = str_detect(title,
                            pattern = "( [Rr] | [Rr]$|^[Rr] )"),
           isPy = str_detect(title,
                             pattern = "\\b[Pp]ython\\b"),
           isRPy = isR & isPy)

# Obtém a soma em cada variável indicadora.
tb_books %>%
    summarise_at(vars(starts_with("is")), "sum")

# Confere alguns títulos.
tb_books %>%
    filter(isRPy) %>%
    select(id, title) %>%
    print(n = Inf)

# Padrão dentro da variável ano.
head(tb_books$date, n = 30)

#-----------------------------------------------------------------------
# Formata a variável ano de publicação.

# Cria a variável ano (opção 1: fatiar).
tb_books <- tb_books %>%
    mutate(year = as.integer(str_sub(date, start = 1, end = 4)))
head(tb_books$year)

# Cria a variável ano (opção 2: por padrão regular).
tb_books <- tb_books %>%
    mutate(year = as.integer(str_replace(date, "^(\\d{4}).*", "\\1")))
head(tb_books$year)

# As datas não estão com padrão. Dá pra preencher se for necessário.
with(head(tb_books, n = 30), {
    data.frame("in" = date,
               "out" = as.Date(date, format = "%Y-%m-%d"))
})

#-----------------------------------------------------------------------
# Quantifica a publicações conforme uma lista de termos.

ds_terms <- c("data", "science",
              "statistic", "probabili", "analysis", "analit", "bayes",
              "model", "methods",
              "regression", "multivar", "survey", "sample", "sampling",
              "visualiz", "graph",
              "finance", "time", "series",
              "mathemat", "spatial", "social",
              "applied", "applicat", "quantitat", "practical",
              "machine", "deep", "learn", "mining", "supervised",
              "artificial", "intelli", "pattern",
              "text", "natural", "language")
ds_pattern <- paste0("(", paste(ds_terms, collapse = "|"), ")")
ds_pattern

pr_terms <- c("algorith", "program", "develop", "build", "hands", "web",
              "implement", "comput", "process", "coding", "code",
              "language", "structures", "raspberry")
pr_pattern <- paste0("(", paste(pr_terms, collapse = "|"), ")")

# Cria uma variável para indicar os termos.
tb_search <- tb_books %>%
    mutate(ds_terms = grepl(x = title,
                 pattern = ds_pattern,
                 ignore.case = TRUE),
           pr_terms = grepl(x = title,
                 pattern = pr_pattern,
                 ignore.case = TRUE))

# Conta as ocorrências por linguagem.
tb_terms <- list()
tb_terms$`Data Science` <- tb_search %>%
    filter(ds_terms) %>%
    count(isR, isPy, year) %>%
    filter(between(year, 2000, 2020),
           xor(isR, isPy)) %>%
    mutate(lang = ifelse(isPy, "Python", "R"))
tb_terms$`Programming` <- tb_search %>%
    filter(pr_terms) %>%
    count(isR, isPy, year) %>%
    filter(between(year, 2000, 2020),
           xor(isR, isPy)) %>%
    mutate(lang = ifelse(isPy, "Python", "R"))

tb_terms <- tb_terms %>%
    bind_rows(.id = "terms")
str(tb_terms)

ggplot(data = tb_terms,
       mapping = aes(x = year, y = n, color = lang)) +
    facet_wrap(facets = ~terms) +
    geom_point() +
    geom_line(lty = 2) +
    scale_y_log10() +
    geom_smooth(method = "lm", se = FALSE, size = 0.5) +
    scale_color_manual(values = c("orange", "blue")) +
    labs(color = "Linguagem",
         x = "Ano",
         y = "Número de livros")

#-----------------------------------------------------------------------
# Faz a núvem de palavras com os títulos.

library(wordcloud)
library(tidytext)

# Separar R e Python para processar a núvem de palavras.
split_books <- list(R = filter(tb_books, isR),
                    Python = filter(tb_books, isPy))
map(split_books, dim)

# A letra R já está nas stop words.
stop_words %>%
    filter(word == "r")

# Adicionar o Python como stop word.
stop_words <- stop_words %>%
    add_row(word = "python")
tail(stop_words)

# Define as cores para usar na núvem.
pal <- brewer.pal(8, "Dark2")

# Processa para ter a tabela de frequência de tokens.
split_tokens <-
    map(split_books,
        function(tb) {
            tokens <- tb %>%
                select(id, title) %>%
                unnest_tokens(output = "word", input = title) %>%
                count(word, sort = TRUE) %>%
                ungroup()
            tokens <- tokens %>%
                anti_join(stop_words) %>%
                filter(str_detect(word, "\\d", negate = TRUE))
            tokens
        })

par(mfrow = c(1, 2))
imap(split_tokens,
     function(tb, element) {
         with(tb, {
             wordcloud(words = word,
                       freq = n,
                       random.order = FALSE,
                       max.words = 500,
                       min.freq = 3,
                       rot.per = 0,
                       scale = c(6, 0.5),
                       colors = pal,
                       family = "Roboto Condensed")
             title(main = element)
         })
     })
layout(1)

#-----------------------------------------------------------------------
# Comparação.

# Junta em uma tabela só. NOTE: trocar NA por 0.
full_tokens <- full_join(rename(split_tokens$R, "R" = "n"),
                         rename(split_tokens$Python, "Python" = "n"),
                         by = "word") %>%
    replace_na(replace = list(R = 0, Python = 0))
head(full_tokens)

# sum(is.na(full_tokens$R))
# sum(is.na(full_tokens$Python))

m <- max(full_tokens[, c("R", "Python")])

gridExtra::grid.arrange(
               ggplot(data = slice_max(full_tokens, order_by = R, n = 50),
                      mapping = aes(x = R, y = reorder(word, R))) +
               geom_col(fill = "blue") +
               xlim(c(0, m)),
               ggplot(data = slice_max(full_tokens, order_by = Python, n = 50),
                      mapping = aes(x = Python, y = reorder(word, Python))) +
               geom_col(fill = "orange") +
               xlim(c(0, m)),
               nrow = 1)

full_tokens %>%
    slice_max(order_by = R + Python, n = 40) %>%
    ggplot(data = .) +
    geom_col(mapping = aes(x = -R, y = reorder(word, R + Python)),
             fill = "blue") +
    geom_col(mapping = aes(x = Python, y = reorder(word, R + Python)),
             fill = "orange") +
    geom_vline(xintercept = 0) +
    geom_label(mapping = aes(y = word,
                            # x = ifelse(Python - R > 0, -R, Python),
                            x = 0,
                            label = word),
               size = 3)

#-----------------------------------------------------------------------

# Cria a matriz para usar na núvem de palavras.
j <- c("R", "Python")
term_matrix <- as.matrix(full_tokens[, j])
rownames(term_matrix) <- full_tokens$word
str(term_matrix)

comparison.cloud(term_matrix,
                 max.words = 300,
                 random.order = FALSE,
                 rot.per = 0,
                 colors = c("blue", "orange"),
                 match.colors = TRUE,
                 title.colors = c("blue", "orange"),
                 family = "Roboto Condensed",
                 scale = c(6, 0.1))

#-----------------------------------------------------------------------
